function [sim_measure]=shapesim(V1,V2,name1,name2)
%
% Compute similarity between two shapes 
%
% USAGE
%
% [sim_measure]=shapesim(V1,V2,name1,name2)
%
% INPUTS
%
% V1, V2: greyscale or binary images of objects to be compared.
%   V1 is viewed as a template/prototype/ground truth, and V2 is
%   to be tested for similarity to V1
% name1, name2: strings that labels the two images (optional)
%
% OUTPUT
%
% sim_measure: a positive real number indicating the similarity
%   between the two shapes.  Smaller means more similar.
%
% NOTES
%
% The original code on which this is based was taken from
% Serge Belongie's web site.
%
% 8/8/02 Clay Scott
%



%
% Parameters I occasionally tweak
%
th = 128;            % threshhold that determines contour
display_flag=1;     % Set to 1 to see diagrams
trans_type=2;       % 1=rigid (trans/rot), 2=linear conformal (trans/rot/scale), 
                    % 3=affine, 4=thin-plate spline (affine + local warping)
nsamp=130;          % number of samples around V1's boundary
n_iter=3;           % number of iterations (updating correspondences and transformation)
lopm_start=Inf;     % look for largest order-preserving matching beyond this iter
assgn_type=1;       % 1 -> order preserving assignment, 0 -> not
copap_delta=1;      % set to 1 for exact solution to COPAP, or > 1 for faster
                    % approx. solution. suggestion: copap_delta = sqrt(nsamp)/2
convex = 0;         % 1 -> object of interest is convex, 0 -> not; limited testing for this feature
tang_meth = 0;      % method for computing tangents: 0 -> from image gradient, 1 -> from boundary points
eps_dum=0.15;       % cost of matching point to dummy point
nondum_frac=.85;    % fraction of samples of source that must be matched to points on target
mean_dist_global=[];    % for shape context scale normalization: 
                        % set equal to [] for scale invariance
ori_weight=0;          % weight given to discrepancy in orientation of objects 
                        % should be small, since we want rotational invariance
                        % set > zero to provide a little "momentum"
prox_weight=0.0;        % weight given to proximity cost: encourage neighbors of corresponding points
                        % to be neighbors.  haven't gotten this to work yet.
prox_width=3;           % ignore for now

%
% Parameters I rarely tweak
%
nbins_theta=12;
nbins_r=5;
r_inner=1/8;
r_outer=2;
tan_eps=1.0;
polarity_flag=1;
affine_start_flag=1;    % make first iter affine when using tps
beta_init=1;    % tps parameter
r=1; % annealing rate (tps)
w=4;        % window (for computing local sums of squared diffs) has width 2*w+1
sf=2.5;     % scale factor by which to enlarge input images
%cmap=flipud(gray);
cmap = gray;


if nargin < 4
    name1='Image 1';
    name2='Image 2';
end


[N1,N2]=size(V1);
[M1,M2]=size(V2);

% normalized intensity
%V1=V1/max(V1(:));
%V2=V2/max(V2(:));

% display images
if display_flag
   figure(1)
   subplot(2,2,1)
   imagesc(V1);axis('image')
   axis off
   title(name1);
   subplot(2,2,2)
   imagesc(V2);axis('image')
   axis off
   title(name2);
   colormap(cmap)
   drawnow
end

%
% Get boundary points
%
[x1,y1,t1]=bdry(V1,th,tang_meth);
[x2,y2,t2]=bdry(V2,th,tang_meth);

% Note from Clay: bdry uses contourc to get the boundary
% pts.  The pts are in 'ij' coordinates, which are like row/column
% coordinates.  x1 and x2 are the column coords, and y1 and y2
% are the row coords.  However, the shape context/transformation 
% code treats these as if they are 'xy' coordinates.  It doesn't 
% affect the outcome since we are consistent.
% Hence, when we display the sampled points
% below, these aren't really what we're working with.
% The only thing to be aware of is that the points being used for
% the computations are actually
% oriented in a counter-clockwise fashion, when viewed in xy coords,
% even though the tanget vectors point in a clockwise orientation.


% ensure clockwise orientation (contourc isn't consistent 
% in this regard).  Need this for COPAP matching.
ang_diff1=diff([t1; t1(1)]);
ang_ind1=find(abs(ang_diff1)>pi);
if sum(sign(ang_diff1(ang_ind1))) < 0
   x1=flipud(x1);
   y1=flipud(y1);
   t1=flipud(t1); 
end
ang_diff2=diff([t2; t2(1)]);
ang_ind2=find(abs(ang_diff2)>pi);
if sum(sign(ang_diff2(ang_ind2))) < 0
   x2=flipud(x2);
   y2=flipud(y2);
   t2=flipud(t2); 
end
   
% select subset of boundary points
nsamp1=length(x1);
nsamp2=length(x2);
if nsamp1*nsamp2==0
  disp('No object in one of the images')
  return;    
end
%frac12 = nsamp2/nsamp1;
frac12=1;
if nsamp2>=round(nsamp*frac12)
    [x2,y2,t2]=get_samples(x2,y2,t2,round(nsamp*frac12));
else
   error('shape #2 doesn''t have enough samples')
end
Y=[x2 y2];
if nsamp1>=nsamp
   [x1,y1,t1]=get_samples(x1,y1,t1,nsamp);
else
   error('shape #1 doesn''t have enough samples')
end
X=[x1 y1];

nsamp1=length(x1);
nsamp2=length(x2);

% display boundary points
if display_flag
   subplot(2,2,3)
   plot(X(:,1),X(:,2),'b+')
   hold on
   %quiver(X(:,1),X(:,2),cos(t1),sin(t1),0.5,'b')
   hold off
   axis('ij');axis([1 N2 1 N1])
   axis off
   title([int2str(length(x1)) ' samples'])
   subplot(2,2,4)
   plot(Y(:,1),Y(:,2),'ro')
   hold on
   %quiver(Y(:,1),Y(:,2),cos(t2),sin(t2),0.5,'r.')
   hold off
   axis('ij');axis([1 N2 1 N1])
   axis off
   title([int2str(length(x2)) ' samples'])
   drawnow	
end

if display_flag
   [x,y]=meshgrid(linspace(1,N2,36),linspace(1,N1,36));
   x=x(:);y=y(:);M=length(x);
end

% Main loop: compute correspondences
Xk=X;
tk=t1;
k=1;    % iteration number
s=1;    % loop while 1
out_vec_1=zeros(1,nsamp1);  % indicates outliers in shape 1
out_vec_2=zeros(1,nsamp2);  % indicates outliers in shape 2
while s % main loop
   itt=cputime;     % iteration time
   disp(['iteration =' int2str(k)])
   
   % compute shape context histograms
   [BH1,mean_dist_1]=sc_compute(Xk',tk',mean_dist_global,nbins_theta,nbins_r,r_inner,r_outer,out_vec_1,convex);
   % apply the scale estimate from the warped model to the test shape
   [BH2,mean_dist_2,r_array_2]=sc_compute(Y',t2',mean_dist_1,nbins_theta,nbins_r,r_inner,r_outer,out_vec_2,convex);
   
   % set tps params
   if affine_start_flag
      if k==1
   	    % use huge regularization to get affine behavior
	    lambda_o=1000;
      else
	    lambda_o=beta_init*r^(k-2);	 
      end
   else
      lambda_o=beta_init*r^(k-1);
   end
   beta_k=(mean_dist_2^2)*lambda_o;
   
   % compute cost matrix using chi-squared distance
   costmat_shape=hist_cost(BH1,BH2);
   %costmat_prox=prox_cost(costmat_shape,prox_width,r_array_2,mean_dist_2);
   theta_diff=repmat(tk,1,nsamp2)-repmat(t2',nsamp1,1);
   if polarity_flag
      % use edge polarity
      costmat_theta=0.5*(1-cos(theta_diff));
   else
      % ignore edge polarity
      costmat_theta=0.5*(1-cos(2*theta_diff));
   end      
   costmat=(1-ori_weight-prox_weight)*costmat_shape+ ...
       ori_weight*costmat_theta;
   % + prox_weight*costmat_prox;
   
   % add dummy vars to cost matrix
   min_match=round(nondum_frac*min(nsamp1,nsamp2)); % min num of required matches
   nptsd=nsamp1+nsamp2-min_match;   % tot num of pts, including dummies
   %   nptsd=max(nsamp1+ndum,nsamp2+ndum);
   costmat2=eps_dum*ones(nptsd,nptsd);
   costmat2(1:nsamp1,1:nsamp2)=costmat;

   % determine correspondences by solving assignment problem
   assgn_time=cputime;     
   if assgn_type == 1
     % optimal order preserving assignment
     [cvec, min_cost]=copap(costmat,min_match,eps_dum,copap_delta);
   else
     [cvec, min_cost]=jvc(costmat2);  % replace hungarian.m with faster jvc.c
   end
   disp(['Assignment time: ' num2str(cputime-assgn_time)])
   
   % update outlier indicator vectors
   [a,cvec2]=sort(cvec);
   out_vec_1=cvec2(1:nsamp1)>nsamp1;
   out_vec_2=cvec(1:nsamp2)>nsamp2;

   X2=NaN*ones(nptsd,2);
   X2(1:nsamp1,:)=Xk;
   X2=X2(cvec,:);           % transformed, sorted source image
   X2b=NaN*ones(nptsd,2);
   X2b(1:nsamp1,:)=X;
   X2b=X2b(cvec,:);         % untransformed, sorted source image
   Y2=NaN*ones(nptsd,2);
   Y2(1:nsamp2,:)=Y;        % target image
   
   % extract coordinates of non-dummy correspondences and use them
   % to estimate transformation
   ind_good=find(~isnan(X2b(1:nsamp2,1)));
   
   % get coordinates of inliers
   n_good=length(ind_good);
   X3b=X2b(ind_good,:);
   Y3=Y2(ind_good,:);

   if display_flag 
      % show the correspondences between the untransformed images
      figure(2)
      plot(X(:,1),X(:,2),'b+',Y(:,1),Y(:,2),'ro')
      ind=cvec(ind_good);
      hold on
      plot([X2b(ind_good,1) Y2(ind_good,1)]',[X2b(ind_good,2) Y2(ind_good,2)]','k-')
      hold off
      axis('ij')
      title([int2str(n_good) ' correspondences (unwarped X)'])
      axis([1 N2 1 N1])
      drawnow
      %pause
   end

   % estimate transformation
   switch trans_type
   case {1 2 3}
       switch trans_type
       case 1
          T=rigid(X3b,Y3);       % translations and rotations only
       case 2
          T=linconf(X3b,Y3);       % translation, rotation, and scaling
       case 3
          T=affine(X3b,Y3);       % arbitrary affine trans.
       end
       % next few lines give above routines same output format as bookstein
       c=zeros(n_good+3,2);
       c(end-2:end,:)=T(:,[3 1 2])';
       cx=c(:,1);
       cy=c(:,2);
       E=0;         % bending energy
   case 4
      [cx,cy,E]=bookstein(X3b,Y3,beta_k); % calculate optimal thin-plate spline
   end
   
   % calculate affine cost
   A=[cx(n_good+2:n_good+3,:) cy(n_good+2:n_good+3,:)];
   s=svd(A);
   aff_cost=log(s(1)/s(2));
   
   % calculate shape context cost
   [a1,b1]=min(costmat,[],1);
   [a2,b2]=min(costmat,[],2);
   sc_cost=max(mean(a1),mean(a2));
   
   % warp each coordinate
   fx_aff=cx(n_good+1:n_good+3)'*[ones(1,nsamp1); X'];
   d2=max(dist2(X3b,X),0);
   U=d2.*log(d2+eps);
   fx_wrp=cx(1:n_good)'*U;
   fx=fx_aff+fx_wrp;
   fy_aff=cy(n_good+1:n_good+3)'*[ones(1,nsamp1); X'];
   fy_wrp=cy(1:n_good)'*U;
   fy=fy_aff+fy_wrp;

   Z=[fx; fy]';

   % apply the warp to the tangent vectors to get the new angles
   Xtan=X+tan_eps*[cos(t1) sin(t1)];
   fx_aff=cx(n_good+1:n_good+3)'*[ones(1,nsamp1); Xtan'];
   d2=max(dist2(X3b,Xtan),0);
   U=d2.*log(d2+eps);
   fx_wrp=cx(1:n_good)'*U;
   fx=fx_aff+fx_wrp;
   fy_aff=cy(n_good+1:n_good+3)'*[ones(1,nsamp1); Xtan'];
   fy_wrp=cy(1:n_good)'*U;
   fy=fy_aff+fy_wrp;
   
   Ztan=[fx; fy]';
   tk=atan2(Ztan(:,2)-Z(:,2),Ztan(:,1)-Z(:,1));

   if display_flag
      figure(3)
      plot(Z(:,1),Z(:,2),'b+',Y(:,1),Y(:,2),'ro');
      axis('ij')
      hold on
	  quiver(Z(:,1),Z(:,2),cos(tk),sin(tk),0.5,'b')
      axis equal
 	  quiver(Y(:,1),Y(:,2),cos(t2),sin(t2),0.5,'r')
      hold off
      title([int2str(length(ind_good)) ' correspondences (transformed point set with tangents)'])
      axis([1 N2 1 N1])
      % show warped coordinate grid
      fx_aff=cx(n_good+1:n_good+3)'*[ones(1,M); x'; y'];
      d2=dist2(X3b,[x y]);
      fx_wrp=cx(1:n_good)'*(d2.*log(d2+eps));
      fx=fx_aff+fx_wrp;
      fy_aff=cy(n_good+1:n_good+3)'*[ones(1,M); x'; y'];
      fy_wrp=cy(1:n_good)'*(d2.*log(d2+eps));
      fy=fy_aff+fy_wrp;
      hold on
      %plot(fx,fy,'k.','markersize',1)
      hold off
      drawnow
   end
   
   Z2=NaN*ones(nptsd,2);
   Z2(1:nsamp1,:)=Z;
   Z2=Z2(cvec,:);           % transformed, sorted source image

   if  display_flag
      figure(4)
      plot(Z(:,1),Z(:,2),'b+',Y(:,1),Y(:,2),'ro')
      hold on
      h=plot([Z2(ind_good,1) Y2(ind_good,1)]',[Z2(ind_good,2) Y2(ind_good,2)]','k-');
	  %quiver(Z(:,1),Z(:,2),cos(tk),sin(tk),0.5,'b')
 	 % quiver(Y(:,1),Y(:,2),cos(t2),sin(t2),0.5,'r')
      hold off
      axis('ij')
      title([int2str(length(ind_good)) ' correspondences (transformed)'])
      axis([1 N2 1 N1])
      drawnow	
   end
      
   % update Xk for the next iteration
   Xk=Z;
  
   if k==n_iter
      s=0;
   else
      k=k+1;
   end
   disp(['Total iteration time ' num2str(cputime-itt)]);

end


% grayscale warping
[x,y]=meshgrid(1:N2,1:N1);
x=x(:);y=y(:);M=length(x);
fx_aff=cx(n_good+1:n_good+3)'*[ones(1,M); x'; y'];
d2=dist2(X3b,[x y]);
fx_wrp=cx(1:n_good)'*(d2.*log(d2+eps));
fx=fx_aff+fx_wrp;
fy_aff=cy(n_good+1:n_good+3)'*[ones(1,M); x'; y'];
fy_wrp=cy(1:n_good)'*(d2.*log(d2+eps));
fy=fy_aff+fy_wrp;
disp('computing warped image...')
V1w=griddata(reshape(fx,N1,N2),reshape(fy,N1,N2),V1,reshape(x,N1,N2),reshape(y,N1,N2));   
fz=find(isnan(V1w)); V1w(fz)=0;
ssd=(V2-V1w).^2;
ssd_global=sum(ssd(:));
if display_flag
   figure(5)
   subplot(2,2,1)
   im(V1)
   subplot(2,2,2)
   im(V2)
   subplot(2,2,4)
   im(V1w,[min(V1(:)) max(V1(:))])
   title('V1 after warping')
   subplot(2,2,3)
   reg_diff=V2-V1w;
   tmp=(V1w~=0);
   im(V2-V1w,[min(reg_diff(tmp)) max(reg_diff(tmp))])
   h=title(['SSD=' num2str(ssd_global)]);
   colormap(cmap)
end

% windowed SSD comparison
wd=2*w+1;
win_fun=fspecial('gaussian',[wd wd],4);
%win_fun=gaussker(wd);
% extract sets of blocks around each coordinate
% first do 1st shape; need to use transformed coords.
win_list_1=zeros(nsamp1,wd^2);
for qq=1:nsamp1
   row_qq=round(Xk(qq,2));
   col_qq=round(Xk(qq,1));
   row_qq=max(w+1,min(N1-w,row_qq));
   col_qq=max(w+1,min(N2-w,col_qq));
   tmp=V1w(row_qq-w:row_qq+w,col_qq-w:col_qq+w);
   tmp=win_fun.*tmp;
   win_list_1(qq,:)=tmp(:)';
end
% now do 2nd shape
win_list_2=zeros(nsamp2,wd^2);
for qq=1:nsamp2
   row_qq=round(Y(qq,2));
   col_qq=round(Y(qq,1));
   row_qq=max(w+1,min(N1-w,row_qq));
   col_qq=max(w+1,min(N2-w,col_qq));
   tmp=V2(row_qq-w:row_qq+w,col_qq-w:col_qq+w);
   tmp=win_fun.*tmp;
   win_list_2(qq,:)=tmp(:)';
end
ssd_all=sqrt(dist2(win_list_1,win_list_2));
if 0
   % visualize paired-up patches
   for qq=1:nsamp
      im([reshape(win_list_1(qq,:),wd,wd) reshape(win_list_2(b2(qq),:),wd,wd); ...
	  reshape(win_list_2(qq,:),wd,wd) reshape(win_list_1(b1(qq),:),wd,wd)])
      colormap(cmap)
      pause
   end
end

% compute shape context and appearance costs
cost_1=0;
cost_2=0;
app_cost=0;
for qq=1:nsamp1
   if out_vec_1(qq)==0
       app_cost=app_cost+ssd_all(qq,b2(qq));
   end
   cost_1=cost_1+costmat(qq,b2(qq));
end
for qq=1:nsamp2
   cost_2=cost_2+costmat(b1(qq),qq);
end

sc_cost=cost_1/nsamp1+cost_2/nsamp2;

% compute similarity measure based on formula from paper: 
% linear combo of "appearance cost", shape context cost, and bending energy.
% note: bending energy is zero unless you are using thin-plate splines
% note: the 1.6 is clearly dependent on the range of pixel values 
% in the images you are considering.
% note: I have normalized the appearance cost by the number of inliers
% and the maximum pixel intensity.

% sim_measure= 1.6*app_cost+sc_cost+.3*E;

% haven't quite gotten the appearance cost to work in a way that makes sense,
% so for now let's go with

sim_measure = min_cost;

if display_flag
   set(h,'string',['Similarity =' num2str(sim_measure)])
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Extract boundary points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [x,y,t,c]=bdry(V,th,tang_meth);
% [x,y,t,c]=bdry(V,th);
% extract a set of boundary points w/oriented tangents

% smooth image. note : smoothing is implicit in gaussian edge detector
if 0
    W=5;    % window width
    h=fspecial('gaussian',W);   % create filter
    Vp=padarray(V,[(W-1)/2,(W-1)/2],'symmetric','both');
    Vg=filter2(h,Vp,'valid');
else
    Vg=V; % if no smoothing is needed
end
c=contourc(Vg,[th th]);
[G1,G2]=gradient(Vg);

% if multiple contours, take largest.
cind=find(c(1,:)==th);
[cnum, kind]=max(c(2,cind));
bind=cind(kind);
B=c(:,bind+1:bind+cnum);

    
x=B(1,:)';
y=B(2,:)';

% compute tangets
npts=size(B,2);
t=zeros(npts,1);
if tang_meth==0
 for n=1:npts
   x0=round(B(1,n)); % consider rounding coord of B to match tan vecs
   y0=round(B(2,n));
   %keyboard;
   t(n)=atan2(G2(y0,x0),G1(y0,x0))+pi/2;
 end
else
  yfor=[y; y(1)];
  xfor=[x; x(1)];
  yback=[y(end); y];
  xback=[x(end); x];
  theta_for=atan2(diff(yfor),diff(xfor));
  theta_back=atan2(diff(yback),diff(xback));
  t=mod(.5*(theta_back+theta_for)+pi*(theta_for.*theta_back >= 0),2*pi);
 % if angles have opposite sign, can't just add them.  must also rotate pi radians.
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Sample boundary points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [xi,yi,ti,ind0]=get_samples(x,y,t,nsamp);
% [xi,yi,ti]=get_samples(x,y,t,nsamp);
%
% Sample boundary points with uniform spacing 

N=length(x);

ind0=1:N/nsamp:N+1;
ind0(end)=[];
ind0=round(ind0);
xi=x(ind0);
yi=y(ind0);
ti=t(ind0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Compute shape context
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [BH,mean_dist,r_array]=sc_compute(Bsamp,Tsamp,mean_dist,nbins_theta,nbins_r,r_inner,r_outer,out_vec,convex);
% [BH,mean_dist]=sc_compute(Bsamp,Tsamp,mean_dist,nbins_theta,nbins_r,r_inner,r_outer,out_vec);
%
% compute (r,theta) histograms for points along boundary 
%
% Bsamp is 2 x nsamp (x and y coords.)
% Tsamp is 1 x nsamp (tangent theta)
% out_vec is 1 x nsamp (0 for inlier, 1 for outlier)
%
% mean_dist is the mean distance, used for length normalization
% if it is not supplied, then it is computed from the data
%
% outliers are not counted in the histograms, but they do get
% assigned a histogram
%

nsamp=size(Bsamp,2);
in_vec=out_vec==0;

% compute r,theta arrays
r_array=real(sqrt(dist2(Bsamp',Bsamp'))); % real is needed to
                                          % prevent bug in Unix version
theta_array_abs=atan2(Bsamp(2,:)'*ones(1,nsamp)-ones(nsamp,1)*Bsamp(2,:),Bsamp(1,:)'*ones(1,nsamp)-ones(nsamp,1)*Bsamp(1,:))';
theta_array=theta_array_abs-Tsamp'*ones(1,nsamp); % orient with respect to tanget vector

% create joint (r,theta) histogram by binning r_array and
% theta_array

% normalize distance by mean, ignoring outliers
if isempty(mean_dist)
   tmp=r_array(in_vec,:);
   tmp=tmp(:,in_vec);
   mean_dist=mean(tmp(:));
end
r_array_n=r_array/mean_dist;

% use a log. scale for binning the distances
r_bin_edges=logspace(log10(r_inner),log10(r_outer),nbins_r);
r_array_q=zeros(nsamp,nsamp);
for m=1:nbins_r
    r_array_q=r_array_q+(r_array_n<r_bin_edges(m)); 
end
fz=r_array_q>0; % flag all points inside outer boundary

% put all angles in [0,2pi) range
theta_array_2 = rem(rem(theta_array,2*pi)+2*pi,2*pi);
% quantize to a fixed set of angles (bin edges lie on 0,(2*pi)/k,...2*pi
theta_array_q = 1+floor(theta_array_2/(2*pi/nbins_theta));

if convex
    fz = fz & ((theta_array_q <= floor(nbins_theta/2) - 1 ) | (theta_array_q == nbins_theta));
end

nbins=nbins_theta*nbins_r;
BH=zeros(nsamp,nbins);
for n=1:nsamp
   fzn=fz(n,:)&in_vec;
   Sn=sparse(theta_array_q(n,fzn),r_array_q(n,fzn),1,nbins_theta,nbins_r);
   BH(n,:)=Sn(:)';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Determine chi-squared distance between s. c. histograms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function HC=hist_cost(BH1,BH2);
% HC=hist_cost(BH1,BH2);
%
% BH1 and BH2 can be of different lengths
%
% computes chi-squared distance between shape context histrograms

[nsamp1,nbins]=size(BH1);
[nsamp2,nbins]=size(BH2);

BH1n=BH1./repmat(sum(BH1,2)+eps,[1 nbins]);
BH2n=BH2./repmat(sum(BH2,2)+eps,[1 nbins]);
tmp1=repmat(permute(BH1n,[1 3 2]),[1 nsamp2 1]);
tmp2=repmat(permute(BH2n',[3 2 1]),[nsamp1 1 1]);
HC=0.5*sum(((tmp1-tmp2).^2)./(tmp1+tmp2+eps),3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Useful function for computing distances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function n2 = dist2(x, c)
%DIST2	Calculates squared distance between two sets of points.
%
%	Description
%	D = DIST2(X, C) takes two matrices of vectors and calculates the
%	squared Euclidean distance between them.  Both matrices must be of
%	the same column dimension.  If X has M rows and N columns, and C has
%	L rows and N columns, then the result has M rows and L columns.  The
%	I, Jth entry is the  squared distance from the Ith row of X to the
%	Jth row of C.
%
%	See also
%	GMMACTIV, KMEANS, RBFFWD
%

%	Copyright (c) Christopher M Bishop, Ian T Nabney (1996, 1997)

[ndata, dimx] = size(x);
[ncentres, dimc] = size(c);
if dimx ~= dimc
	error('dist2.m: Data dimension does not match dimension of centres')
end

n2 = (ones(ncentres, 1) * sum((x.^2)', 1))' + ...
  		ones(ndata, 1) * sum((c.^2)',1) - ...
  		2.*(x*(c'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Useful function for displaying images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function im(X,vec);
% im(X) or im(X,[minval maxval])
%
% Handy function for showing a grayscale image with a colorbar and
% interactive pixel value tool
%
% Serge Belongie
% 20-Aug-2000
% sjb@eecs.berkeley.edu
 
if nargin==1
   imagesc(X)
else
   imagesc(X,vec)
end
pixval on
title(inputname(1))
colormap(gray)
colorbar
axis('image')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Solve for least squares rigid transform
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%rigid.m
%6/20/02

function [T,E] = rigid(x,y)
% Find least-squares rigid aligning transform.
% A rigid transform is a composition of
% translation, rotation, and reflection.
% Method is based on Schonemann's solution to
% the "orthogonal Procrustes problem". (1966)
%
% x = Nx2, y = Nx2, T = 2x3 (last column = translation)
N=size(x,1);
xbar = mean(x);
ybar = mean(y);
x0=x-repmat(xbar,[N,1]);
y0=y-repmat(ybar,[N,1]);
[U,D,V]=svd(y0'*x0);
R=U*V;		  		%rotation
t=ybar'-R*xbar';	%translation
T=[R t];
D = y - [x ones(N,1)]*T';
E = sum(D(:).^2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Solve for least squares linear conformal transform
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [T,E]=linconf(x,y)
% Find optimal linear conformal (translation, rotation, and/or scaling) 
% transformation for aligning x,y.  A conformal map preserves angles, so 
% a linear conformal map is a linear angle preserving map.
% Also called Helmert transformation (in geodetic sciences),
% similarity mapping (triangles map to similar triangles), 
% or homothetic transformation
N=size(x,1);
X1=x(:,1);
X2 = x(:,2);
P=[X1 X2 ones(N,1) zeros(N,1); X2 -X1 zeros(N,1) ones(N,1)];
Q = reshape(y,[2*N,1]);
R = P\Q;
T = [R(1) R(2) R(3); -R(2) R(1) R(4)];
D = y - [x ones(N,1)]*T';
E = sum(D(:).^2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Solve for least squares affine transform
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [T,E] = affine(x,y);
% Find least-squares optimal affine aligning transformation
%
% x = Nx2, y = Nx2, T = 2x3 (last column = translation)

N=size(x,1);
P=[x'; ones(1,N)];
Q=y';
T=(P'\Q')';
D = Q-T*P;
E = sum(D(:).^2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% newfun: Solve for thin-plate spline interpolating transform
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [cx,cy,E,L]=bookstein(X,Y,beta_k);
% [cx,cy,E,L]=bookstein(X,Y,beta_k);
%
% Bookstein PAMI89

N=size(X,1);
Nb=size(Y,1);

if N~=Nb
   error('number of landmarks must be equal')
end

% compute distances between left points
r2=dist2(X,X);

K=r2.*log(r2+eye(N,N)); % add identity matrix to make K zero on the diagonal
P=[ones(N,1) X];
L=[K  P
   P' zeros(3,3)];
V=[Y' zeros(2,3)];
if nargin>2
   % regularization
   L(1:N,1:N)=L(1:N,1:N)+beta_k*eye(N,N);
end
invL=inv(L);

c=invL*V';
cx=c(:,1);
cy=c(:,2);

if nargout>2
   % compute bending energy (w/o regularization)
   Q=c(1:N,:)'*K*c(1:N,:);
   E=mean(diag(Q));
end
